/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "thumbstick.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define THUMBSTICK_DUMMY 0
#define THUMBSTICK_MARGIN 1365

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void thumbstick_cfg_setup ( thumbstick_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->int_pin = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

THUMBSTICK_RETVAL thumbstick_init ( thumbstick_t *ctx, thumbstick_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return THUMBSTICK_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, THUMBSTICK_DUMMY );

    // Input pins

    digital_in_init( &ctx->int_pin, cfg->int_pin );

    return THUMBSTICK_OK;
}

void thumbstick_generic_transfer ( thumbstick_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);
}

uint16_t thumbstick_read_data ( thumbstick_t *ctx, uint8_t channel )
{
    uint8_t tx_buf[ 3 ];
    uint8_t rx_buf[ 3 ];

    spi_master_transfer_data_t block;
    uint16_t result;

    result = 0x00;
    tx_buf[ 0 ] = 0x06;
    tx_buf[ 1 ] = channel;

    block.tx_buffer = tx_buf;
    block.tx_length = 2;
    block.rx_buffer = rx_buf;
    block.rx_length = 3;

    thumbstick_generic_transfer ( ctx, &block );

    result = rx_buf[ 1 ] & 0x0F;
    result <<= 8;
    result |= rx_buf[ 2 ];

    return result;
}

uint8_t thumbstick_get_position ( thumbstick_t *ctx )
{
    uint16_t ver;
    uint16_t hor;
    uint8_t pos;

    ver  = thumbstick_read_data( ctx, THUMBSTICK_CHANNEL_V );
    hor  = thumbstick_read_data( ctx, THUMBSTICK_CHANNEL_H );

    pos = THUMBSTICK_POSITION_START;

    if ( ( ver < THUMBSTICK_MARGIN ) && ( hor > THUMBSTICK_MARGIN ) && ( hor < ( 2 * THUMBSTICK_MARGIN) ) )
    {
        pos = THUMBSTICK_POSITION_TOP;
    }
    else if ( ( ver > ( 2 * THUMBSTICK_MARGIN ) ) && ( hor > THUMBSTICK_MARGIN ) && ( hor < ( 2 * THUMBSTICK_MARGIN ) ) )
    {
        pos = THUMBSTICK_CHANNEL_BOTTOM;
    }
    else if ( hor < THUMBSTICK_MARGIN )
    {
        pos = THUMBSTICK_CHANNEL_LEFT;
    }
    else if ( hor > ( 2 * THUMBSTICK_MARGIN ) )
    {
        pos = THUMBSTICK_POSITION_RIGHT;
    }
    return pos;
}

uint8_t thumbstick_button_state ( thumbstick_t *ctx )
{
    return digital_in_read( &ctx->int_pin );
}

// ------------------------------------------------------------------------- END

